@@ -33,21 +33,23 @@
 #include <qtextstream.h>
 
 #include <kdebug.h>
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <kstandarddirs.h>
 #include <kconfig.h>
-#include <ksimpleconfig.h>
+#include <kconfiggroup.h>
 #include <klocale.h>
 #include <kprotocolinfo.h>
+#include <kio/global.h>
 #include <kio/slaveconfig.h>
+#include <kconfiggroup.h>
 
-#include <config.h>
+#include "config-kamera.h"
 
 #include "kamera.h"
 
-#define MAXIDLETIME	30	/* seconds */
+#define tocstr(x) ((x).toLocal8Bit())
 
-#define tocstr(x) ((x).local8Bit())
+#define MAXIDLETIME   30      /* seconds */
 
 using namespace KIO;
 
@@ -67,19 +69,22 @@ extern "C"
 
 int kdemain(int argc, char **argv)
 {
-	KInstance	instance("kio_kamera");
+	KComponentData componentData("kio_kamera");
 
 	if(argc != 4) {
-		kdDebug(7123) << "Usage: kio_kamera protocol "
+		kDebug(7123) << "Usage: kio_kamera protocol "
 			     "domain-socket1 domain-socket2" << endl;
 		exit(-1);
 	}
+
 	KameraProtocol slave(argv[2], argv[3]);
+
 	slave.dispatchLoop();
+
 	return 0;
 }
 
-KameraProtocol::KameraProtocol(const QCString &pool, const QCString &app)
+KameraProtocol::KameraProtocol(const QByteArray &pool, const QByteArray &app)
 : SlaveBase("camera", pool, app),
 m_camera(NULL)
 {
@@ -87,11 +92,11 @@ m_camera(NULL)
 	// (will init m_camera, since the m_camera's configuration is empty)
 	m_camera = 0;
 	m_file = NULL;
-	m_config = new KSimpleConfig(KProtocolInfo::config("camera"));
+	m_config = new KConfig(KProtocolInfo::config("camera"), KConfig::SimpleConfig);
 	m_context = gp_context_new();
 	actiondone = true;
 	cameraopen = false;
-	m_lockfile = locateLocal("tmp", "kamera");
+	m_lockfile = KStandardDirs::locateLocal("tmp", "kamera");
 	idletime = 0;
 }
 
@@ -105,12 +110,12 @@ m_camera(NULL)
 // The existance of a lockfile is used to signify "please give up camera".
 //
 void KameraProtocol::special(const QByteArray&) {
-	kdDebug(7123) << "KameraProtocol::special() at " << getpid() << endl;
+	kDebug(7123) << "KameraProtocol::special() at " << getpid();
 
 	if (!actiondone && cameraopen) {
 		struct stat	stbuf;
-		if ((-1!=::stat(m_lockfile.utf8(),&stbuf)) || (idletime++ >= MAXIDLETIME)) {
-			kdDebug(7123) << "KameraProtocol::special() closing camera." << endl;
+		if ((-1!=::stat(m_lockfile.toUtf8(),&stbuf)) || (idletime++ >= MAXIDLETIME)) {
+			kDebug(7123) << "KameraProtocol::special() closing camera.";
 			closeCamera();
 			setTimeoutSpecialCommand(-1);
 		} else {
@@ -126,7 +131,7 @@ void KameraProtocol::special(const QByteArray&) {
 
 KameraProtocol::~KameraProtocol()
 {
-	kdDebug(7123) << "KameraProtocol::~KameraProtocol()" << endl;
+	kDebug(7123) << "KameraProtocol::~KameraProtocol()";
 	delete m_config;
 	if(m_camera) {
 		closeCamera();
@@ -144,25 +149,25 @@ bool KameraProtocol::openCamera(QString &str) {
 	} else {
 		if (!cameraopen) {
 			int ret, tries = 15;
-			kdDebug(7123) << "KameraProtocol::openCamera at " << getpid() << endl;
+			kDebug(7123) << "KameraProtocol::openCamera at " << getpid();
 			while (tries--) {
 				ret = gp_camera_init(m_camera, m_context);
 				if (	(ret == GP_ERROR_IO_USB_CLAIM) || 
 					(ret == GP_ERROR_IO_LOCK)) {
 					// just create / touch if not there
-					int fd = ::open(m_lockfile.utf8(),O_CREAT|O_WRONLY,0600);
+					int fd = ::open(m_lockfile.toUtf8(),O_CREAT|O_WRONLY,0600);
 					if (fd != -1) ::close(fd);
 					::sleep(1);
-					kdDebug(7123) << "openCamera at " << getpid() << "- busy, ret " << ret << ", trying again." << endl;
+					kDebug(7123) << "openCamera at " << getpid() << "- busy, ret " << ret << ", trying again.";
 					continue;
 				}
 				if (ret == GP_OK) break;
 				str = gp_result_as_string(ret);
 				return false;
 			}
-			::unlink(m_lockfile.utf8());
+			::unlink(m_lockfile.toUtf8());
 			setTimeoutSpecialCommand(1);
-			kdDebug(7123) << "openCamera succeeded at " << getpid() << endl;
+			kDebug(7123) << "openCamera succeeded at " << getpid();
 			cameraopen = true;
 		}
 	}
@@ -177,9 +182,9 @@ void KameraProtocol::closeCamera(void)
 	if (!m_camera)
 		return;
 
-	kdDebug(7123) << "KameraProtocol::closeCamera at " << getpid() << endl;
+	kDebug(7123) << "KameraProtocol::closeCamera at " << getpid();
 	if ((gpr=gp_camera_exit(m_camera,m_context))!=GP_OK) {
-		kdDebug(7123) << "closeCamera failed with " << gp_result_as_string(gpr) << endl;
+		kDebug(7123) << "closeCamera failed with " << gp_result_as_string(gpr);
 	}
 	// HACK: gp_camera_exit() in gp 2.0 does not close the port if there
 	//       is no camera_exit function.
@@ -201,9 +206,9 @@ static QString fix_foldername(QString ofolder) {
 
 // The KIO slave "get" function (starts a download from the camera)
 // The actual returning of the data is done in the frontend callback functions.
-void KameraProtocol::get(const KURL &url)
+void KameraProtocol::get(const KUrl &url)
 {
-	kdDebug(7123) << "KameraProtocol::get(" << url.path() << ")" << endl;
+	kDebug(7123) << "KameraProtocol::get(" << url.path() << ")";
 
 	CameraFileType fileType;
 	int gpr;
@@ -217,7 +222,7 @@ void KameraProtocol::get(const KURL &url)
 		return;
 	}
 
-	// fprintf(stderr,"get(%s)\n",url.path().latin1());
+	// fprintf(stderr,"get(%s)\n",url.path().toLatin1());
 
 #define GPHOTO_TEXT_FILE(xx)						\
 	if (!url.path().compare("/" #xx ".txt")) {			\
@@ -227,11 +232,10 @@ void KameraProtocol::get(const KURL &url)
 			error(KIO::ERR_DOES_NOT_EXIST, url.path());	\
 			return;						\
 		}							\
-		QByteArray chunkDataBuffer;				\
-		chunkDataBuffer.setRawData(xx.text, strlen(xx.text));	\
+		QByteArray chunkDataBuffer = QByteArray::fromRawData(xx.text, strlen(xx.text));	\
 		data(chunkDataBuffer);					\
 		processedSize(strlen(xx.text));				\
-		chunkDataBuffer.resetRawData(xx.text, strlen(xx.text));	\
+		chunkDataBuffer.clear();                                \
 		finished();						\
 		return;							\
 	}
@@ -242,7 +246,7 @@ void KameraProtocol::get(const KURL &url)
 
 #undef GPHOTO_TEXT_FILE
 	// emit info message
-	infoMessage( i18n("Retrieving data from camera <b>%1</b>").arg(url.user()) );
+	infoMessage( i18n("Retrieving data from camera <b>%1</b>", url.user()) );
 
 	// Note: There's no need to re-read directory for each get() anymore
 	gp_file_new(&m_file);
@@ -250,9 +254,9 @@ void KameraProtocol::get(const KURL &url)
 	// emit the total size (we must do it before sending data to allow preview)
 	CameraFileInfo info;
 	
-	gpr = gp_camera_file_get_info(m_camera, tocstr(fix_foldername(url.directory(false))), tocstr(url.fileName()), &info, m_context);
+	gpr = gp_camera_file_get_info(m_camera, tocstr(fix_foldername(url.directory(KUrl::AppendTrailingSlash))), tocstr(url.fileName()), &info, m_context);
 	if (gpr != GP_OK) {
-		// fprintf(stderr,"Folder %s / File %s not found, gpr is %d\n",folder.latin1(), url.fileName().latin1(), gpr);
+		// fprintf(stderr,"Folder %s / File %s not found, gpr is %d\n",folder.toLatin1(), url.fileName().toLatin1(), gpr);
 		gp_file_unref(m_file);
 		if ((gpr == GP_ERROR_FILE_NOT_FOUND) || (gpr == GP_ERROR_DIRECTORY_NOT_FOUND))
 			error(KIO::ERR_DOES_NOT_EXIST, url.path());
@@ -263,14 +267,14 @@ void KameraProtocol::get(const KURL &url)
 
 	// at last, a proper API to determine whether a thumbnail was requested.
 	if(cameraSupportsPreview() && metaData("thumbnail") == "1") {
-		kdDebug(7123) << "get() retrieving the thumbnail" << endl;
+		kDebug(7123) << "get() retrieving the thumbnail";
 		fileType = GP_FILE_TYPE_PREVIEW;
 		if (info.preview.fields & GP_FILE_INFO_SIZE)
 			totalSize(info.preview.size);
 		if (info.preview.fields & GP_FILE_INFO_TYPE)
 			mimeType(info.preview.type);
 	} else {
-		kdDebug(7123) << "get() retrieving the full-scale photo" << endl;
+		kDebug(7123) << "get() retrieving the full-scale photo";
 		fileType = GP_FILE_TYPE_NORMAL;
 		if (info.file.fields & GP_FILE_INFO_SIZE)
 			totalSize(info.file.size);
@@ -280,14 +284,14 @@ void KameraProtocol::get(const KURL &url)
 
 	// fetch the data
 	m_fileSize = 0;
-	gpr = gp_camera_file_get(m_camera, tocstr(fix_foldername(url.directory(false))), tocstr(url.fileName()), fileType, m_file, m_context);
+	gpr = gp_camera_file_get(m_camera, tocstr(fix_foldername(url.directory(KUrl::AppendTrailingSlash))), tocstr(url.fileName()), fileType, m_file, m_context);
 	if (	(gpr == GP_ERROR_NOT_SUPPORTED) &&
 		(fileType == GP_FILE_TYPE_PREVIEW)
 	) {
 		// If we get here, the file info command information 
 		// will either not be used, or still valid.
 		fileType = GP_FILE_TYPE_NORMAL;
-		gpr = gp_camera_file_get(m_camera, tocstr(fix_foldername(url.directory(false))), tocstr(url.fileName()), fileType, m_file, m_context);
+		gpr = gp_camera_file_get(m_camera, tocstr(fix_foldername(url.directory(KUrl::AppendTrailingSlash))), tocstr(url.fileName()), fileType, m_file, m_context);
 	}
 	switch(gpr) {
 		case GP_OK:
@@ -318,7 +322,7 @@ void KameraProtocol::get(const KURL &url)
 	// buffer -- there's no expensive memcpy
 	gpr = gp_file_get_data_and_size(m_file, &fileData, &fileSize);
 	if (gpr != GP_OK) {
-		kdDebug(7123) << "get():: get_data_and_size failed." << endl;
+		kDebug(7123) << "get():: get_data_and_size failed.";
 		gp_file_free(m_file);
 		m_file = NULL;
 		error(KIO::ERR_UNKNOWN, gp_result_as_string(gpr));
@@ -340,10 +344,10 @@ void KameraProtocol::get(const KURL &url)
 
 			if (towrite > fileSize-m_fileSize-written)
 				towrite = fileSize-m_fileSize-written;
-			chunkDataBuffer.setRawData(fileData + m_fileSize + written, towrite);
+			chunkDataBuffer = QByteArray::fromRawData(fileData + m_fileSize + written, towrite);
 			processedSize(m_fileSize + written + towrite);
 			data(chunkDataBuffer);
-			chunkDataBuffer.resetRawData(fileData + m_fileSize + written, towrite);
+			chunkDataBuffer.clear();
 			written += towrite;
 		}
 		m_fileSize = fileSize;
@@ -356,14 +360,14 @@ void KameraProtocol::get(const KURL &url)
 }
 
 // The KIO slave "stat" function.
-void KameraProtocol::stat(const KURL &url)
+void KameraProtocol::stat(const KUrl &url)
 {
-	kdDebug(7123) << "stat(\"" << url.path() << "\")" << endl;
+	kDebug(7123) << "stat(\"" << url.path() << "\")";
 	
-	if (url.path() == "") {
-		KURL rooturl(url);
+	if (url.path().isEmpty()) {
+		KUrl rooturl(url);
 
-		kdDebug(7123) << "redirecting to /" << endl;
+		kDebug(7123) << "redirecting to /";
 		rooturl.setPath("/");
 		rooturl.setHost(url.host());
 		rooturl.setUser(url.user());
@@ -381,51 +385,42 @@ void KameraProtocol::stat(const KURL &url)
 // Implements stat("/") -- which always returns the same value.
 void KameraProtocol::statRoot(void)
 {
-	UDSEntry entry;
- 	UDSAtom atom;
+	KIO::UDSEntry entry;
 
-	atom.m_uds = UDS_NAME;
-	atom.m_str = "/";
-	entry.append(atom);
+	entry.insert( KIO::UDSEntry::UDS_NAME, QString::fromLocal8Bit("/"));
 
-	atom.m_uds = UDS_FILE_TYPE;
-	atom.m_long = S_IFDIR;
-	entry.append(atom);
+	entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,S_IFDIR);
 
-	atom.m_uds = UDS_ACCESS;
-	atom.m_long = S_IRUSR | S_IRGRP | S_IROTH |
-			S_IWUSR | S_IWGRP | S_IWOTH;
-	entry.append(atom);
+	entry.insert(KIO::UDSEntry::UDS_ACCESS,(S_IRUSR | S_IRGRP | S_IROTH |S_IWUSR | S_IWGRP | S_IWOTH));
 
 	statEntry(entry);
 
 	finished();
-
-	// This call happens on autodetect by kdemm. So close the camera, but
-	// only if no more requests are pending. 
+	// If we just do this call, timeout right away if no other requests are
+	// pending. This is for the kdemm autodetection using media://camera
 	idletime = MAXIDLETIME;
 }
 
 // Implements a regular stat() of a file / directory, returning all we know about it
-void KameraProtocol::statRegular(const KURL &url)
+void KameraProtocol::statRegular(const KUrl &url)
 {
-	UDSEntry entry;
+	KIO::UDSEntry entry;
 	int gpr;
 
-	kdDebug(7123) << "statRegular(\"" << url.path() << "\")" << endl;
+	kDebug(7123) << "statRegular(\"" << url.path() << "\")";
 	if (openCamera() == false) {
 		error(KIO::ERR_DOES_NOT_EXIST, url.path());
 		return;
 	}
 
-	// fprintf(stderr,"statRegular(%s)\n",url.path().latin1());
+	// fprintf(stderr,"statRegular(%s)\n",url.path().toLatin1());
 
 	// Is "url" a directory?
 	CameraList *dirList;
 	gp_list_new(&dirList);
-	kdDebug(7123) << "statRegular() Requesting directories list for " << url.directory() << endl;
+	kDebug(7123) << "statRegular() Requesting directories list for " << url.directory();
 
-	gpr = gp_camera_folder_list_folders(m_camera, tocstr(fix_foldername(url.directory(false))), dirList, m_context);
+	gpr = gp_camera_folder_list_folders(m_camera, tocstr(fix_foldername(url.directory(KUrl::AppendTrailingSlash))), dirList, m_context);
 	if (gpr != GP_OK) {
 		if ((gpr == GP_ERROR_FILE_NOT_FOUND) || (gpr == GP_ERROR_DIRECTORY_NOT_FOUND))
 			error(KIO::ERR_DOES_NOT_EXIST, url.path());
@@ -458,7 +453,7 @@ void KameraProtocol::statRegular(const KURL &url)
 		gp_list_get_name(dirList, i, &name);
 		if (url.fileName().compare(name) == 0) {
 			gp_list_free(dirList);
-			UDSEntry entry;
+			KIO::UDSEntry entry;
 			translateDirectoryToUDS(entry, url.fileName());
 			statEntry(entry);
 			finished();
@@ -469,7 +464,7 @@ void KameraProtocol::statRegular(const KURL &url)
 
 	// Is "url" a file?
 	CameraFileInfo info;
-	gpr = gp_camera_file_get_info(m_camera, tocstr(fix_foldername(url.directory(false))), tocstr(url.fileName()), &info, m_context);
+	gpr = gp_camera_file_get_info(m_camera, tocstr(fix_foldername(url.directory(KUrl::AppendTrailingSlash))), tocstr(url.fileName()), &info, m_context);
 	if (gpr != GP_OK) {
 		if ((gpr == GP_ERROR_FILE_NOT_FOUND) || (gpr == GP_ERROR_DIRECTORY_NOT_FOUND))
 			error(KIO::ERR_DOES_NOT_EXIST, url.path());
@@ -483,9 +478,9 @@ void KameraProtocol::statRegular(const KURL &url)
 }
 
 // The KIO slave "del" function.
-void KameraProtocol::del(const KURL &url, bool isFile)
+void KameraProtocol::del(const KUrl &url, bool isFile)
 {
-	kdDebug(7123) << "KameraProtocol::del(" << url.path() << ")" << endl;
+	kDebug(7123) << "KameraProtocol::del(" << url.path() << ")";
 
 	if(!openCamera()) {
 		error(KIO::ERR_CANNOT_DELETE, url.fileName());
@@ -500,7 +495,7 @@ void KameraProtocol::del(const KURL &url, bool isFile)
 		gp_list_new(&list);
 		int ret;
 
-		ret = gp_camera_file_delete(m_camera, tocstr(fix_foldername(url.directory(false))), tocstr(url.fileName()), m_context);
+		ret = gp_camera_file_delete(m_camera, tocstr(fix_foldername(url.directory(KUrl::AppendTrailingSlash))), tocstr(url.fileName()), m_context);
 
 		if(ret != GP_OK) {
 			error(KIO::ERR_CANNOT_DELETE, url.fileName());
@@ -511,18 +506,17 @@ void KameraProtocol::del(const KURL &url, bool isFile)
 }
 
 // The KIO slave "listDir" function.
-void KameraProtocol::listDir(const KURL &url)
+void KameraProtocol::listDir(const KUrl &url)
 {
-	kdDebug(7123) << "KameraProtocol::listDir(" << url.path() << ")" << endl;
+	kDebug(7123) << "KameraProtocol::listDir(" << url.path() << ")";
 
 	if (url.host().isEmpty()) {
-		KURL xurl;
+		KUrl xurl;
 		// List the available cameras
 		QStringList groupList = m_config->groupList();
-		kdDebug(7123) << "Found cameras: " << groupList.join(", ") << endl;
+		kDebug(7123) << "Found cameras: " << groupList.join(", ");
 		QStringList::Iterator it;
-		UDSEntry entry;
-		UDSAtom atom;
+		KIO::UDSEntry entry;
 
 
 		/*
@@ -560,15 +554,15 @@ void KameraProtocol::listDir(const KURL &url)
 	
 			ports[value] = model;
 			// NOTE: We might get different ports than usb: later!
-			if (strcmp(value, "usb:"))
+			if (strcmp(value,"usb:"))
 				names[model] = value;
 
 			/* Save them, even though we can autodetect them for
 			 * offline listing.
 			 */
-			m_config->setGroup(model);
-			m_config->writeEntry("Model",model);
-			m_config->writeEntry("Path",value);
+			KConfigGroup cg(m_config, model);
+			cg.writeEntry("Model", model);
+			cg.writeEntry("Path", value);
 			modelcnt[model]++;
 		}
 		gp_list_free (list);
@@ -582,8 +576,8 @@ void KameraProtocol::listDir(const KURL &url)
 			if (*it == "<default>")
 				continue;
 
-			m_config->setGroup(*it);
-			m_cfgPath = m_config->readEntry("Path");
+			KConfigGroup cg(m_config, *it);
+			m_cfgPath = cg.readEntry("Path");
 
 			/* If autodetect by USB autodetect ... skip it here.
 			 * We leave unattached USB cameras in here, because the user
@@ -594,15 +588,11 @@ void KameraProtocol::listDir(const KURL &url)
 				continue;
 
 			entry.clear();
-			atom.m_uds = UDS_FILE_TYPE;atom.m_long = S_IFDIR;entry.append(atom);
-			atom.m_uds = UDS_NAME;atom.m_str = *it;entry.append(atom);
-			atom.m_uds = UDS_ACCESS;
-			atom.m_long = S_IRUSR | S_IRGRP | S_IROTH |
-				S_IWUSR | S_IWGRP | S_IWOTH;
-			entry.append(atom);
-
-			atom.m_uds = UDS_URL;
+			entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,S_IFDIR);
+			entry.insert(KIO::UDSEntry::UDS_NAME,*it);
+			entry.insert(KIO::UDSEntry::UDS_ACCESS,(S_IRUSR | S_IRGRP | S_IROTH |S_IWUSR | S_IWGRP | S_IWOTH));
 
+			
 			xurl.setProtocol("camera");
 			xurl.setUser(*it);
 			/* Avoid setting usb:xxx,yyy. */
@@ -613,8 +603,7 @@ void KameraProtocol::listDir(const KURL &url)
 				xurl.setHost(m_cfgPath);
 			}
 			xurl.setPath("/");
-			atom.m_str = xurl.url();
-			entry.append(atom);
+			entry.insert(KIO::UDSEntry::UDS_URL,xurl.url());
 
 			listEntry(entry, false);
 		}
@@ -623,21 +612,16 @@ void KameraProtocol::listDir(const KURL &url)
 
 		for (portsit = ports.begin(); portsit != ports.end(); portsit++) {
 			entry.clear();
-			atom.m_uds = UDS_FILE_TYPE;atom.m_long = S_IFDIR; entry.append(atom);
-			atom.m_uds = UDS_NAME;atom.m_str = portsit.data();entry.append(atom);
+			entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,S_IFDIR);
+			entry.insert(KIO::UDSEntry::UDS_NAME, portsit.value());
 
-			atom.m_uds = UDS_ACCESS;
-			atom.m_long = S_IRUSR | S_IRGRP | S_IROTH |
-				S_IWUSR | S_IWGRP | S_IWOTH;
-			entry.append(atom);
+			entry.insert(KIO::UDSEntry::UDS_ACCESS,(S_IRUSR | S_IRGRP | S_IROTH |S_IWUSR | S_IWGRP | S_IWOTH));
 
-			atom.m_uds = UDS_URL;
 			xurl.setProtocol("camera");
 			xurl.setHost(portsit.key());
-			xurl.setUser(portsit.data());
+			xurl.setUser(portsit.value());
 			xurl.setPath("/");
-			atom.m_str = xurl.url();
-			entry.append(atom);
+			entry.insert(KIO::UDSEntry::UDS_URL,xurl.url());
 
 			listEntry(entry, false);
 		}
@@ -647,10 +631,10 @@ void KameraProtocol::listDir(const KURL &url)
 		return;
 	}
 
-	if (url.path() == "") {
-		KURL rooturl(url);
+	if (url.path().isEmpty()) {
+		KUrl rooturl(url);
 
-		kdDebug(7123) << "redirecting to /" << endl;
+		kDebug(7123) << "redirecting to /";
 		rooturl.setPath("/");
 		rooturl.setHost(url.host());
 		rooturl.setUser(url.user());
@@ -684,7 +668,7 @@ void KameraProtocol::listDir(const KURL &url)
 
 	gpr = readCameraFolder(url.path(), dirList, fileList);
 	if(gpr != GP_OK) {
-		kdDebug(7123) << "read Camera Folder failed:" << gp_result_as_string(gpr) <<endl;
+		kDebug(7123) << "read Camera Folder failed:" << gp_result_as_string(gpr);
 		gp_list_free(dirList);
 		gp_list_free(fileList);
 		gp_list_free(specialList);
@@ -694,7 +678,7 @@ void KameraProtocol::listDir(const KURL &url)
 
 	totalSize(gp_list_count(specialList) + gp_list_count(dirList) + gp_list_count(fileList));
 
-	UDSEntry entry;
+	KIO::UDSEntry entry;
 	const char *name;
 
 	for(int i = 0; i < gp_list_count(dirList); ++i) {
@@ -737,21 +721,21 @@ void KameraProtocol::listDir(const KURL &url)
 	finished();
 }
 
-void KameraProtocol::setHost(const QString& host, int port, const QString& user, const QString& pass )
+void KameraProtocol::setHost(const QString& host, quint16 port, const QString& user, const QString& pass )
 {
-	kdDebug(7123) << "KameraProtocol::setHost(" << host << ", " << port << ", " << user << ", " << pass << ")" << endl;
+	kDebug(7123) << "KameraProtocol::setHost(" << host << ", " << port << ", " << user << ", " << pass << ")";
 	int gpr, idx;
 
 	if (!host.isEmpty()) {
-		kdDebug(7123) << "model is " << user << ", port is " << host << endl;
+		kDebug(7123) << "model is " << user << ", port is " << host;
 		if (m_camera) {
-			kdDebug(7123) << "Configuration change detected" << endl;
+			kDebug(7123) << "Configuration change detected";
 			closeCamera();
 			gp_camera_unref(m_camera);
 			m_camera = NULL;
 			infoMessage( i18n("Reinitializing camera") );
 		} else {
-			kdDebug(7123) << "Initializing camera" << endl;
+			kDebug(7123) << "Initializing camera";
 			infoMessage( i18n("Initializing camera") );
 		}
 		// fetch abilities
@@ -761,7 +745,7 @@ void KameraProtocol::setHost(const QString& host, int port, const QString& user,
 		idx = gp_abilities_list_lookup_model(abilities_list, tocstr(user));
 		if (idx < 0) {
 			gp_abilities_list_free(abilities_list);
-			kdDebug(7123) << "Unable to get abilities for model: " << user << endl;
+			kDebug(7123) << "Unable to get abilities for model: " << user;
 			error(KIO::ERR_UNKNOWN, gp_result_as_string(idx));
 			return;
 		}
@@ -780,7 +764,7 @@ void KameraProtocol::setHost(const QString& host, int port, const QString& user,
 			idx = gp_port_info_list_lookup_path(port_info_list, "usb:");
 		if (idx < 0) {
 			gp_port_info_list_free(port_info_list);
-			kdDebug(7123) << "Unable to get port info for path: " << host << endl;
+			kDebug(7123) << "Unable to get port info for path: " << host;
 			error(KIO::ERR_UNKNOWN, gp_result_as_string(idx));
 			return;
 		}
@@ -803,13 +787,15 @@ void KameraProtocol::setHost(const QString& host, int port, const QString& user,
 		gp_camera_set_abilities(m_camera, m_abilities);
 		gp_camera_set_port_info(m_camera, port_info);
 		gp_camera_set_port_speed(m_camera, 0); // TODO: the value needs to be configurable
-		kdDebug(7123) << "Opening camera model " << user << " at " << host << endl;
-
+		kDebug(7123) << "Opening camera model " << user << " at " << host;
+		
 		QString errstr;
 		if (!openCamera(errstr)) {
-			kdDebug(7123) << "Unable to init camera: " << gp_result_as_string(gpr) << endl;
+			if (m_camera)
+				gp_camera_unref(m_camera);
+			m_camera = NULL;
+			kDebug(7123) << "Unable to init camera: " << errstr;
 			error(KIO::ERR_SERVICE_NOT_AVAILABLE, errstr);
-			gp_camera_exit(m_camera, m_context);
 			return;
 		}
 	}
@@ -821,110 +807,67 @@ void KameraProtocol::reparseConfiguration(void)
 }
 
 // translate a simple text to a UDS entry
-void KameraProtocol::translateTextToUDS(UDSEntry &udsEntry, const QString &fn,
+void KameraProtocol::translateTextToUDS(KIO::UDSEntry &udsEntry, const QString &fn,
 	const char *text
 ) {
-	UDSAtom atom;
 
 	udsEntry.clear();
 
-	atom.m_uds = UDS_FILE_TYPE; // UDS type
-	atom.m_long = S_IFREG; // file
-	udsEntry.append(atom);
+	udsEntry.insert(KIO::UDSEntry::UDS_FILE_TYPE,S_IFREG);
 
-	atom.m_uds = UDS_NAME;
-	atom.m_str = fn;
-	udsEntry.append(atom);
+	udsEntry.insert(KIO::UDSEntry::UDS_NAME,fn);
 
-	atom.m_uds = UDS_SIZE;
-	atom.m_long = strlen(text);
-	udsEntry.append(atom);
+	udsEntry.insert(KIO::UDSEntry::UDS_SIZE,strlen(text));
 
-	atom.m_uds = UDS_ACCESS;
-	atom.m_long = S_IRUSR | S_IRGRP | S_IROTH;
-	udsEntry.append(atom);
+	udsEntry.insert(KIO::UDSEntry::UDS_ACCESS,(S_IRUSR | S_IRGRP | S_IROTH));
 }
 
-// translate a CameraFileInfo to a UDSEntry which we can return as a directory listing entry
-void KameraProtocol::translateFileToUDS(UDSEntry &udsEntry, const CameraFileInfo &info, QString name)
+// translate a CameraFileInfo to a UDSFieldType which we can return as a directory listing entry
+void KameraProtocol::translateFileToUDS(KIO::UDSEntry &udsEntry, const CameraFileInfo &info, QString name)
 {
-	UDSAtom atom;
 
 	udsEntry.clear();
 
-	atom.m_uds = UDS_FILE_TYPE; // UDS type
-	atom.m_long = S_IFREG; // file
-	udsEntry.append(atom);
+	udsEntry.insert(KIO::UDSEntry::UDS_FILE_TYPE,S_IFREG);
 
-	atom.m_uds = UDS_NAME;
 	if (info.file.fields & GP_FILE_INFO_NAME)
-		atom.m_str = QString::fromLocal8Bit(info.file.name);
+			udsEntry.insert(KIO::UDSEntry::UDS_NAME,QString::fromLocal8Bit(info.file.name));
 	else
-		atom.m_str = name;
-	udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_NAME,name);
 
 	if (info.file.fields & GP_FILE_INFO_SIZE) {
-		atom.m_uds = UDS_SIZE;
-		atom.m_long = info.file.size;
-		udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_SIZE,info.file.size);
 	}
 
 	if (info.file.fields & GP_FILE_INFO_MTIME) {
-		atom.m_uds = UDS_MODIFICATION_TIME;
-		atom.m_long = info.file.mtime;
-		udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME,info.file.mtime);
 	} else {
-		atom.m_uds = UDS_MODIFICATION_TIME;
-		atom.m_long = time(NULL); /* NOW */
-		udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME,time(NULL));
 	}
 
 	if (info.file.fields & GP_FILE_INFO_TYPE) {
-		atom.m_uds = UDS_MIME_TYPE;
-		atom.m_str = QString::fromLatin1(info.file.type);
-		udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_MIME_TYPE,QString::fromLatin1(info.file.type));
 	}
 
 	if (info.file.fields & GP_FILE_INFO_PERMISSIONS) {
-		atom.m_uds = UDS_ACCESS;
-		atom.m_long = 0;
-		atom.m_long |= (info.file.permissions & GP_FILE_PERM_READ) ? (S_IRUSR | S_IRGRP | S_IROTH) : 0;
-		// we cannot represent individual FP_FILE_PERM_DELETE permission in the Unix access scheme
-		// since the parent directory's write permission defines that
-		udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_ACCESS,((info.file.permissions & GP_FILE_PERM_READ) ? (S_IRUSR | S_IRGRP | S_IROTH) : 0));
 	} else {
-		// basic permissions, in case the camera doesn't provide permissions info
-		atom.m_uds = UDS_ACCESS;
-		atom.m_long = S_IRUSR | S_IRGRP | S_IROTH;
-		udsEntry.append(atom);
+		udsEntry.insert(KIO::UDSEntry::UDS_ACCESS,S_IRUSR | S_IRGRP | S_IROTH);
 	}
 
 	// TODO: We do not handle info.preview in any way
 }
 
-// translate a directory name to a UDSEntry which we can return as a directory listing entry
-void KameraProtocol::translateDirectoryToUDS(UDSEntry &udsEntry, const QString &dirname)
+// translate a directory name to a UDSFieldType which we can return as a directory listing entry
+void KameraProtocol::translateDirectoryToUDS(KIO::UDSEntry &udsEntry, const QString &dirname)
 {
-	UDSAtom atom;
 
 	udsEntry.clear();
 
-	atom.m_uds = UDS_FILE_TYPE; // UDS type
-	atom.m_long = S_IFDIR; // directory
-	udsEntry.append(atom);
-
-	atom.m_uds = UDS_NAME;
-	atom.m_str = dirname;
-	udsEntry.append(atom);
-
-	atom.m_uds = UDS_ACCESS;
-	atom.m_long = S_IRUSR | S_IRGRP | S_IROTH |
-			S_IWUSR | S_IWGRP | S_IWOTH;
-	udsEntry.append(atom);
-
-        atom.m_uds = UDS_MIME_TYPE;
-        atom.m_str = "inode/directory";
-        udsEntry.append(atom);
+	udsEntry.insert(KIO::UDSEntry::UDS_FILE_TYPE,S_IFDIR);
+	udsEntry.insert(KIO::UDSEntry::UDS_NAME,dirname);
+	udsEntry.insert(KIO::UDSEntry::UDS_ACCESS,S_IRUSR | S_IRGRP | S_IROTH |S_IWUSR | S_IWGRP | S_IWOTH);
+        udsEntry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString("inode/directory"));
 }
 
 bool KameraProtocol::cameraSupportsDel(void)
@@ -944,7 +887,7 @@ bool KameraProtocol::cameraSupportsPreview(void)
 
 int KameraProtocol::readCameraFolder(const QString &folder, CameraList *dirList, CameraList *fileList)
 {
-	kdDebug(7123) << "KameraProtocol::readCameraFolder(" << folder << ")" << endl;
+	kDebug(7123) << "KameraProtocol::readCameraFolder(" << folder << ")";
 
 	int gpr;
 	if((gpr = gp_camera_folder_list_folders(m_camera, tocstr(folder), dirList, m_context)) != GP_OK)
@@ -975,12 +918,11 @@ void frontendProgressUpdate(
 	if (fileSize > 0) {
 		// XXX using assign() here causes segfault, prolly because
 		// gp_file_free is called before chunkData goes out of scope
-		QByteArray chunkDataBuffer;
-		chunkDataBuffer.setRawData(fileData + object->getFileSize(), fileSize - object->getFileSize());
+		QByteArray chunkDataBuffer = QByteArray::fromRawData(fileData + object->getFileSize(), fileSize - object->getFileSize());
 		// Note: this will fail with sizes > 16MB ... 
 		object->data(chunkDataBuffer);
 		object->processedSize(fileSize);
-		chunkDataBuffer.resetRawData(fileData + object->getFileSize(), fileSize - object->getFileSize());
+		chunkDataBuffer.clear();
 		object->setFileSize(fileSize);
 	}
 }
