@@ -18,23 +18,19 @@
 #ifndef KLINEAL_H
 #define KLINEAL_H
 
-#include <kapplication.h>
-#include <kpopupmenu.h>
-#include <kmainwindow.h>
+#include <QWidget>
 
 #include <kcolordialog.h>
 
-#include <qlabel.h>
-#include <qpainter.h>
-#include <qwidget.h>
-#include <qcursor.h>
+class QLabel;
+class KMenu;
 
-class KLineal : public KMainWindow {
-  Q_OBJECT 
-public: 
+class KLineal : public QWidget {
+  Q_OBJECT
+public:
   enum { North=0, West=1, South=2, East=3 };
   /** constructor */
-  KLineal(QWidget*parent=0,const char* name=0);
+  KLineal(QWidget*parent=0);
   /** destructor */
   ~KLineal();
   void move(int x, int y);
@@ -43,28 +39,31 @@ public:
   int x();
   int y();
 protected:
-	void keyPressEvent(QKeyEvent *e);
+  void keyPressEvent(QKeyEvent *e);
   void mousePressEvent(QMouseEvent *e);
-  void mouseReleaseEvent(QMouseEvent *e); 
+  void mouseReleaseEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
-  void paintEvent(QPaintEvent *e); 
-  void enterEvent(QEvent *e); 
-  void leaveEvent(QEvent *e); 
+  void paintEvent(QPaintEvent *e);
+  void enterEvent(QEvent *e);
+  void leaveEvent(QEvent *e);
+
 
 private:
   void drawScale(QPainter &painter);
+  void drawBackground(QPainter &painter);
   void reLength(int percentOfScreen);
   bool mDragging;
-	QIconSet menuIcon(const char *name);
   QPoint mLastClickPos;
   QPoint mDragOffset;
   QLabel *mLabel;
   QLabel *mColorLabel;
-  QFrame *mColorRect;
   int mOrientation;
   int mLongEdgeLen;
   int mShortEdgeLen;
-  KPopupMenu *mMenu;
+  KMenu *mMenu;
+  KMenu *mLenMenu;
+  QAction *mFullScreenAction;
+  QColor mColor;
   QColor mStoredColor;
   QCursor mCurrentCursor;
   QCursor mNorthCursor;
@@ -73,7 +72,8 @@ private:
   QCursor mSouthCursor;
   QCursor mDragCursor;
   KColorDialog mColorSelector;
-	bool _clicked;
+  QFont mScaleFont;
+  bool _clicked;
 public slots:
   void setOrientation(int);
   void setNorth();
@@ -91,9 +91,12 @@ public slots:
   void setTallLength();
   void setFullLength();
   void setColor();
+  void setFont(QFont &);
   void setColor(const QColor &color);
   void choseColor();
+  void choseFont();
   void restoreColor();
-  void saveColor();
+  void saveSettings();
+  void slotQuit();
 };
 #endif
